@@ -20,11 +20,6 @@
 # 02110-1301, USA.
 #
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import errno
 import optparse
 import os
@@ -248,6 +243,11 @@ match the namespace prefix.""")
     parser.add_option("", "--include-last-in-src",
                       action="append", dest="include_last_src", default=[],
                       help="Header to include after the other headers in generated sources")
+    parser.add_option("", "--sources-top-dirs", default=[], action='append',
+                      help="Paths to the sources directories used to determine"
+                      " relative files locations to be used in the gir file."
+                      " This is especially useful when build dir and source dir are different"
+                      " and mirrored.")
 
     return parser
 
@@ -439,9 +439,13 @@ def create_source_scanner(options, args):
                        options.cpp_defines,
                        options.cpp_undefines,
                        cflags=options.cflags)
-    ss.parse_files(filenames)
-    ss.parse_macros(filenames)
-    return ss
+    try:
+        ss.parse_files(filenames)
+        ss.parse_macros(filenames)
+    finally:
+        for error in ss.get_errors():
+            print(error, file=sys.stderr)
+    return ss, filenames
 
 
 def write_output(data, options):
@@ -535,7 +539,7 @@ def scanner_main(args):
         except pkgconfig.PkgConfigError as e:
             _error(str(e))
 
-    ss = create_source_scanner(options, args)
+    ss, filenames = create_source_scanner(options, args)
 
     cbp = GtkDocCommentBlockParser()
     blocks = cbp.parse_comment_blocks(ss.get_comments())
@@ -575,7 +579,9 @@ def scanner_main(args):
 
     transformer.namespace.c_includes = options.c_includes
     transformer.namespace.exported_packages = exported_packages
-    writer = Writer(transformer.namespace)
+    if not options.sources_top_dirs:
+        options.sources_top_dirs = [os.path.commonprefix(filenames).rpartition(os.path.sep)[0]]
+    writer = Writer(transformer.namespace, options.sources_top_dirs)
     data = writer.get_encoded_xml()
 
     write_output(data, options)
